@@ -53,14 +53,13 @@ def apache_conf_path():
     return os.path.join(beanstalk_stack_base, 'confs/', 'apache.conf')
 
 
-def apps_pool_path():
+def app_ports_path():
     beanstalk_settings = env.get('beanstalk_settings', None)
     if beanstalk_settings is None:
         abort('Load beanstalk settings first')
 
     beanstalk_stack_base = beanstalk_settings['BEANSTALK_STACK_BASE']
-    return os.path.join(beanstalk_stack_base, 'apps/', 'apps.json')
-
+    return os.path.join(beanstalk_stack_base, 'confs/', 'ports.json')
 
 # Tasks
 #-----------------------------------------------------------------------------------------------------------------------
@@ -139,6 +138,14 @@ def setup(settings_path=None, **settings_patches):
         local('chmod 644 ~beanstalk/.ssh/authorized_keys')
         local('chown -R beanstalk:beanstalk ~beanstalk/.ssh')
 
+    # Create homebase
+    print section_title('Deploy target is: %s' % beanstalk_stack_base)
+    local('mkdir -p %s' % beanstalk_stack_base)
+    local('mkdir -p %s' % os.path.join(beanstalk_stack_base, 'global-logs'))
+    local('mkdir -p %s' % os.path.join(beanstalk_stack_base, 'confs'))
+    local('echo "[]" > %s' % app_ports_path())
+    local('chmod o+rx %s' % beanstalk_stack_base)
+
     # Apache command
     print section_title('Create apache-reload command')
     httpd_cmd_path = '/usr/sbin/httpd_reload'
@@ -159,16 +166,7 @@ def setup(settings_path=None, **settings_patches):
     local('echo "# Beanstalk Apache conf" > %s' % apache_conf_path())
     local('chown beanstalk:beanstalk %s' % apache_conf_path())
 
-    # Create homebase
-    print section_title('Deploy target is: %s' % beanstalk_stack_base)
-    local('mkdir -p %s' % beanstalk_stack_base)
-    local('mkdir -p %s' % os.path.join(beanstalk_stack_base, 'global-logs'))
-    local('mkdir -p %s' % os.path.join(beanstalk_stack_base, 'apps'))
-    local('mkdir -p %s' % os.path.join(beanstalk_stack_base, 'confs'))
-    local('echo "{}" >> %s' % apps_pool_path())
-    local('chmod o+rx %s' % beanstalk_stack_base)
     local('chown -R beanstalk:beanstalk %s' % beanstalk_stack_base)
-
     local('/etc/init.d/sshd restart')
     local('/etc/init.d/httpd reload')
 
@@ -213,48 +211,34 @@ def create_app(project_name, settings_path=None, **settings_patches):
             local('git add .gitignore')
             local('git commit -m "Init commit"')
 
-    # Apps pool
-    with open(apps_pool_path(), 'r') as f:
-        apps_pool = json.loads(f.read())
-
     # Get uwsgi port
-    uwsgi_port = None
-    if project_name in apps_pool:
-        uwsgi_port = apps_pool[project_name]['uwsgi']['port']
+    with open(app_ports_path(), 'r') as f:
+        ports = json.loads(f.read())
+    if len(ports) == 0:
+        uwsgi_port = 50000
     else:
-        app_ports = [app['uwsgi']['port'] for name, app in apps_pool.items()]
-        if len(app_ports) == 0:
-            uwsgi_port = 50000  # >= 49152
-        else:
-            try_times = 1
-            while try_times <= 10:
-                try:
-                    uwsgi_port = max(app_ports) + try_times
-                    s = socket.socket()
-                    s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
-                    s.bind(('localhost', uwsgi_port))
-                    break
-                except socket.error:
-                    uwsgi_port = None
-                    try_times += 1
-    if uwsgi_port is None:
-        abort('No available port')
-
-    def editor(pool):
-        pool[project_name] = {
-            'path': {
-                'base': project_base,
-                'source': project_source,
-                'logs': project_logs,
-                'venv': project_venv,
-            },
-            'uwsgi': {
-                'port': uwsgi_port,
-            },
-        }
-        return pool
-    print section_title('Add app to app_pool')
-    edit_file(apps_pool_path(), editor=editor)
+        uwsgi_port = max(ports) + 1
+    # Available ports?
+    try_times = 1
+    port_available = False
+    while try_times <= 10:
+        try:
+            s = socket.socket()
+            s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
+            s.bind(('localhost', uwsgi_port))
+            s.close()
+            port_available = True
+            break
+        except socket.error:
+            uwsgi_port += 1
+            try_times += 1
+    if not port_available:
+        abort('No available port for uwsgi')
+    else:
+        ports.append(uwsgi_port)
+    # Save it
+    with open(app_ports_path(), 'w') as f:
+        f.write(json.dumps(ports, separators=(',', ':')))
 
     print section_title('Add to apache httpd')
     with open(apache_conf_path(), 'r') as f:
@@ -273,6 +257,9 @@ def create_app(project_name, settings_path=None, **settings_patches):
         sed_command = '\\\n'.join(commands)
         local('sed -i.bak -e \'$a\\%s\' %s && rm -rf %s.bak' % (sed_command, apache_conf_path(), apache_conf_path()))
 
+    port_file_path = os.path.join(project_base_path(project_name), '%d.port' % uwsgi_port)
+    local('echo "%d" > %s' % (uwsgi_port, port_file_path))
+
 
 @task
 @beanstalk_role('server')
@@ -291,13 +278,18 @@ def delete_app(project_name, settings_path=None, **settings_patches):
     # delete
     if remove:
         print section_title('Remove app @ %s' % project_base)
-        local('rm -rf %s' % project_base)
+        try:
+            port_number = int([name for name in os.listdir(project_base) if name.endswith('.port')][0].split('.')[0])
+            with open(app_ports_path(), 'r') as f:
+                ports = json.loads(f.read())
+            if port_number in ports:
+                ports.remove(port_number)
+            with open(app_ports_path(), 'w') as f:
+                f.write(json.dumps(ports))
+        except (IndexError, ValueError):
+            pass
 
-        def editor(apps_pool):
-            if project_name in apps_pool:
-                del apps_pool[project_name]
-            return apps_pool
-        edit_file(apps_pool_path(), editor)
+        local('rm -rf %s' % project_base)
 
     local('sed -i.bak -e \'/# %s/,+7d\' %s && rm -rf %s.bak' % (project_name, apache_conf_path(), apache_conf_path()))
 
@@ -371,21 +363,33 @@ def app_info(project_name, settings_path=None, **settings_patches):
     beanstalk_settings = load_beanstalk_settings(local_settings_path=settings_path, **settings_patches)
     set_verbose_level()
 
-    with open(apps_pool_path(), 'r') as f:
-        apps_pool = json.loads(f.read())
-
     # Gather information
-    if project_name in apps_pool:
-        apps = apps_pool[project_name]
-    else:
+    apps = os.listdir(os.path.join(beanstalk_settings['BEANSTALK_STACK_BASE'], 'apps'))
+
+    if project_name not in apps:
         if beanstalk_settings.get('USE_JSON', False):
             print json.dumps({'error': 'No such app'})
         else:
             print red('No such app')
         return
 
+    try:
+        uwsgi_port = int([name for name in os.listdir(project_base_path(project_name))
+                          if name.endswith('.port')][0].split('.')[0])
+    except (IndexError, ValueError):
+        abort('No port number!?')
+        return
+
     if beanstalk_settings.get('USE_JSON', False):
-        result = apps
+        result = {
+            'base': project_base_path(project_name),
+            'source': project_source_path(project_name),
+            'logs': project_logs_path(project_name),
+            'venv': project_venv_path(project_name),
+            'uwsgi': {
+                'port': uwsgi_port,
+            },
+        }
         json_string = json.dumps(result, separators=(',', ':'))
 
         if beanstalk_settings.get('PRINT_JSON', True):
@@ -394,12 +398,12 @@ def app_info(project_name, settings_path=None, **settings_patches):
     else:
         print section_title('Beanstalk-stack app: %s' % project_name)
         print '-=' * 40
-        print 'Base   : %s' % apps['path']['base']
-        print 'Source : %s' % apps['path']['source']
-        print 'Logs   : %s' % apps['path']['logs']
-        print 'venv   : %s' % apps['path']['venv']
+        print 'Base   : %s' % project_base_path(project_name)
+        print 'Source : %s' % project_source_path(project_name)
+        print 'Logs   : %s' % project_logs_path(project_name)
+        print 'venv   : %s' % project_venv_path(project_name)
         print '-' * 40
-        print 'uwsgi port : %s' % apps['uwsgi']['port']
+        print 'uwsgi port : %s' % uwsgi_port
 
 
 @task
@@ -408,12 +412,10 @@ def list_apps(settings_path=None, **settings_patches):
     """Get list of apps
     """
     settings_patches = eval_kwargs(settings_patches)
-    load_beanstalk_settings(local_settings_path=settings_path, **settings_patches)
+    beanstalk_settings = load_beanstalk_settings(local_settings_path=settings_path, **settings_patches)
     set_verbose_level()
 
-    with open(apps_pool_path(), 'r') as f:
-        apps_pool = json.loads(f.read())
-    apps = [key for key, value in apps_pool.items()]
+    apps = os.listdir(os.path.join(beanstalk_settings['BEANSTALK_STACK_BASE'], 'apps/'))
 
     print section_title('Beanstalk-stack apps')
     print '-=' * 40
